;;******************************************************************************
;;FILE:               pic-merge-diff3.scmh
;;LANGUAGE:           scheme/gimp
;;SYSTEM:             gimp-1.2
;;USER-INTERFACE:     None
;;DESCRIPTION
;;
;;	  This  program is  a script-fu  gimp program  to merge  the changes
;;    concurently applied to a picture.
;;
;;
;;                             pic1
;;                            / |  \
;;                      edit2/  |   \edit3
;;                          /   |    \
;;                         |    |     |
;;                         V    |     V
;;                        pic2  |    pic3
;;                         |    |     |
;;                         V    V     V
;;                    +---------------------+
;;                    |   pic-merge-diff3   |
;;                    +---------------------+
;;                              |
;;                              V
;;                            merge
;;
;;
;;USAGE
;;    Use the sh script to call this script-fu function in batch mode:
;;
;;    pic-merge-diff3 usage:
;;        pic-merge-diff3 [-h|--help] [-] picture1 picture2 picture3 merge
;;
;;    Or invoke it from the Script-Fu/Utils menu.
;;   
;;AUTHORS
;;    <PJB> Pascal J. Bourguignon
;;MODIFICATIONS
;;    2001-07-14 <PJB> Creation.
;;BUGS
;;
;;    Only single layer pictures are merged: before processing the 
;;    given picture are flattened.
;; 
;;    It is not expected that the pictures change size.
;;    pic1-pic2 is computed with the same size as pic1.
;;    The result pic3-(pic1-pic2) is computed with the same size as pic3.
;;  
;;    Nothing is done to try to compute "conflicts".
;;
;;LEGAL
;;    LGPL
;;    Copyright Pascal J. Bourguignon 2001 - 2001
;;
;;    This  library is  free software;  you can  redistribute  it and/or
;;    modify  it under  the  terms  of the  GNU  Library General  Public
;;    License  as  published by  the  Free  Software Foundation;  either
;;    version 2 of the License, or (at your option) any later version.
;;
;;    This library  is distributed in the  hope that it  will be useful,
;;    but  WITHOUT ANY WARRANTY;  without even  the implied  warranty of
;;    MERCHANTABILITY or FITNESS FOR  A PARTICULAR PURPOSE.  See the GNU
;;    Library General Public License for more details.
;;
;;    You should have received a  copy of the GNU Library General Public
;;    License  along with this  library; see  the file  COPYING.LIB.  If
;;    not,  write to  the Free  Software Foundation,  59 Temple  Place -
;;    Suite 330, Boston, MA 02111-1307, USA.
;;******************************************************************************


(define (copy-and-paste-into-layer  src-image src-layer dst-image dst-layer)
  ;; copy the layer src-layer from the 
  (gimp-image-set-active-layer src-image src-layer)
  (gimp-image-set-active-layer dst-image dst-layer)
  (gimp-selection-all  dst-image)
  (gimp-edit-clear     (car (gimp-image-active-drawable dst-image)))
  (gimp-selection-none dst-image)
  (gimp-selection-all  src-image)
  (gimp-edit-copy (car (gimp-image-active-drawable src-image)))
  (let ((floating-sel 
         (car (gimp-edit-paste 
               (car (gimp-image-active-drawable dst-image)) FALSE))))
    (gimp-floating-sel-anchor floating-sel))
  (gimp-selection-none src-image))


(define (pic-merge-diff3-internal run-mode picf1 picf2 picf3 merged)

  ;;
  ;; Compute the merged image as picf3-(picf1-picf2) = picf3+picf2-picf1
  ;; Each image is first flattened in case where several layers exist.
  ;; The result is a flattened image with the same size than picf3.
  ;;
  ;; BUGS: picf1-picf2 is computed with the same size as picf1.
  ;;       The result picf3-(picf1-picf2) is computed with the same size 
  ;;       as picf3.
  ;;       Nothing is done to try to compute "conflicts".
  ;;

    (let  ((image1 (car (gimp-file-load run-mode picf1 picf1)))
           (image2 (car (gimp-file-load run-mode picf2 picf2)))
           (image3 (car (gimp-file-load run-mode picf3 picf3)))
           (layer1) (layer2) (layer3) (layer1diff2) (layer1diff2in3)
           )

      ;; Debug:
      ;; (gimp-display-new image1)
      ;; (gimp-display-new image2)
      ;; (gimp-display-new image3)

      ;; Sorry, for now, we're going to merge only single layer pictures.
      (set! layer1 (car (gimp-image-flatten image1)))
      (set! layer2 (car (gimp-image-flatten image2)))
      (set! layer3 (car (gimp-image-flatten image3)))


      ;; 1.1- Let's add a DIFFERENCE layer in image1.
      (set! layer2in1 (car (gimp-layer-new image1
                                           (car (gimp-image-width image1))
                                           (car (gimp-image-height image1))
                                           RGB-IMAGE "image 2" 100.0
                                           DIFFERENCE-MODE)))
      (gimp-image-add-layer image1 layer2in1 -1)

      ;; 1.2- Let's copy the image2 into the new layer in image1.
      (copy-and-paste-into-layer image2 layer2 image1 layer2in1)

      ;; 1.3- Let's flatten the image1 to get the difference 
      ;;      between image2 and image1.
      (set! layer1diff2 (car (gimp-image-flatten image1)))

      ;; 2.1- Let's add a DIFFERENCE layer in image3.
      (set! layer1diff2in3 (car (gimp-layer-new image3
                                                (car (gimp-image-width image3))
                                                (car (gimp-image-height image3))
                                                RGB-IMAGE "1 diff 2" 100.0
                                                DIFFERENCE-MODE)))
      (gimp-image-add-layer image3 layer1diff2in3 -1)

      ;; 2.2- Let's copy the image1 (=image1-image2) 
      ;;      into the new layer of image3.
      (copy-and-paste-into-layer image1 layer1diff2 image3 layer1diff2in3)

      ;; 2.3- Let's flatten the result.
      (set! layer3 (car (gimp-image-flatten image3)))

      ;; 3- Let's save the result.
      (gimp-file-save run-mode image3 layer3 merged merged)

      nil
    ))





;;
;; Gimp Glue:
;;


(define (pic-merge-diff3 pic1 pic2 pic3 merged)
  ;; Interactive entry point.
  (pic-merge-diff3-internal RUN-INTERACTIVE pic1 pic2 pic3 merged))


(script-fu-register "pic-merge-diff3"
      _"<Toolbox>/Script-Fu/Utils/Merge Diff3"
      "Merge changes made separately to the same base picture."
      "Pascal J. Bourguignon <pjb@informatimago.com>"
      "Pascal J. Bourguignon <pjb@informatimago.com>"
      "2001"
      ""
      SF-FILENAME _"IN: Original File" ""
      SF-FILENAME _"IN: First Edit to Original File" ""
      SF-FILENAME _"IN: Second Edit to Original File" ""
      SF-FILENAME _"OUT: Merged File" ""
)


;; (define (pic-merge-diff3-test)
;;   ;; Test driver.
;;   (pic-merge-diff3-internal RUN-INTERACTIVE
;;                             "/tmp/tests/tetes/tete1.png"
;;                             "/tmp/tests/tetes/tete2.png"
;;                             "/tmp/tests/tetes/tete3.png"
;;                             "/tmp/tests/tetes/merge.png"))
;; 
;; (script-fu-register "pic-merge-diff3-test"
;;       _"<Toolbox>/Script-Fu/Utils/Merge Diff3 Test"
;;       "Merge changes made separately to the same base picture."
;;       "Pascal J. Bourguignon <pjb@informatimago.com>"
;;       "Pascal J. Bourguignon <pjb@informatimago.com>"
;;       "2001"
;;       ""
;; )

